function out = robustness_func (weeks,robust,round,IFR_robust,rel_rho_robust,psi_robust,sigma_robust,gamma_robust,R0_robust,rU_start_robust)
% Given weeks between tests and other robustness calibration, this file
% will produce the ingredients for plotting robustness figures.
% We consider the following robustness exercise:
% 1. IFR:      consider 1.5% and 0.5%  (original = 1%)
% 2. rel_rho : consider a value of 0.5 (original = 1)
% 3. psi:      consider 0.75 and 0.5   (original = 0.6)  
% 4. gamma:    consider +2/-2 days     (original = 8)
% 5. sigma:    consider +2/-2 days     (original = 6)
% 6. R0:       consider 2 and 3        (original = 2.6)
% 7. rU_start:  consider 0.5/1.5 * baseline level   (original = 0.00275)


global hrs disimm rel_rho R0 lambda rel_lambda
global Pop ini_inf T_years  T  d_vaccine  rel_prod_Q
global sigma gamma tau psi d_conv V_false_neg S_false_neg
global rU_start 

%__________________________________________________________________________
% Time step
hrs                     = 14;               % Periods per day    
%__________________________________________________________________________
% Robustness calibration (ceteris paribus)
if robust == 1
    IFR       = IFR_robust;
else
    IFR       = 0.01;
end

if robust == 2
    rel_rho   = rel_rho_robust;
else
    rel_rho   = 1;
end

if robust == 3
    psi       = psi_robust;
else
    psi       = 0.6   ;
end

if robust == 4
    gamma     = gamma_robust;
else
    gamma     = hrs*8 ;   
end

if robust == 5
    sigma     = sigma_robust;
else
    sigma     = hrs*6 ;
end

if robust == 6
    R0        = R0_robust;
else
    R0        = 2.6 ;
end

if robust == 7
    rU_start   = rU_start_robust;
else
    rU_start   = 0.00275 ;    
end

%__________________________________________________________________________
%
display(robust)
display(round)

%__________________________________________________________________________
% Policy parameters - Fixed
reopen_date             = datetime(2020,6,1);
days_Mar20_to_reopen    = days(reopen_date-datetime(2020,3,20));

testing_date            = datetime(2020,6,1);
days_Mar20_to_testing   = days(testing_date-datetime(2020,3,20));

xiU_c                   = 0.0026;       % constant Q rate for U  % Daily rate
rU_exp                  = 0 ; 

vaccine_date            = datetime(2021,3,20);
days_Mar20_to_vaccine   = days(vaccine_date-datetime(2020,3,20));

% tests has no false negatives for baseline
V_false_neg             = 0;
S_false_neg             = 0; 
%__________________________________________________________________________
% Quarantine technology
rel_lambda              = 0.16 ;       % lambdaQ/lambda

%__________________________________________________________________________
% Parameters
rel_prod_Q              = 0.9;             % Relative productivity of quarantined (A_rel in paper), [BOXES]
disimm                  = 0;                % Dis-Immunity  - GUESS (0 = complete immunity, 1 = no immunity)


lambda                  = 1;                    % Meeting rate (its just a normalization)
Pop                     = 340000000;            % <Background>
T_years                 = 5;                    % <Background>
T                       = T_years*365*hrs;      % <Background>
ini_inf                 = 1*340000;           % 0.1% infected
%__________________________________________________________________________
% Bisection stuff
% d_conv is the date when we define 'long term' for target deaths
d_conv       = 1000;


death_inflator          = 1.33;  % Jones Villaverde
deaths_March20          = 277*death_inflator;

%% Solve for omegaD that delivers the desired infection_fatality rate
infection_fatality_target = IFR;

Pt              = zeros(13,13);
Pt(3,6)         = 1/sigma;
Pt(4,6)         = 1/sigma;
Pt(5,6)         = 1/sigma;
Pt(6,12)        = 1/gamma;
Pt(7,10)        = 1/(sigma+gamma);
Pt(8,11)        = 1/(sigma+gamma);
Pt(9,12)        = 1/(sigma+gamma);
M0              = zeros(13,1);
M0(3)           = psi;
M0(7)           = (1-psi);

maxiter_piD     = 1000;
tol_piD         = 1e-9;
omegaD_min      = 0.0;
omegaD_max      = 1-1/gamma-1e-8;

for iter_omegaD     = (1:maxiter_piD)
    omegaD          = 0.5*(omegaD_min + omegaD_max);
    Pt(6,13)        = omegaD;
    Pdeath          = Pt + diag(1-sum(Pt,2)); 
    ergodic_dist    = (Pdeath')^10000*M0;    
    % All infections
    death_rate_true =  ergodic_dist(13);

    dist_piD        = abs(death_rate_true-infection_fatality_target);
    if dist_piD<tol_piD
        break;
    end
    if (death_rate_true < infection_fatality_target)
        omegaD_min  = omegaD;
    else
        omegaD_max  = omegaD;
    end
    
end

display(['Infection Fatality Rate: ' num2str(death_rate_true) ' : diff_piD : ' num2str(dist_piD) ])
display(['Infection Fatality Rate Percent: ' num2str(100*death_rate_true) ])

fprintf('omegaD = %1.4e\n',omegaD);

%% Set up parameters

model.omegaD      = omegaD;
model.lambdaQ     = rel_lambda*lambda;
model.rhoS        = R0/(psi*sigma*lambda*(rel_rho+(1/sigma)/((1/gamma)+model.omegaD)) + (1-psi)*(sigma+gamma)*lambda*rel_rho) ;
model.rhoA        = rel_rho* model.rhoS; 
 

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %1. Compute Tstar in Quarantine Only Environment
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Run it with no policy to get tstar!!
    Tstar_guess             = 365;                          % Big guess for the first iteration, who cares
    
    d_vaccine               = days_Mar20_to_vaccine*hrs+ Tstar_guess*hrs;     % vaccine available date     
    
    % no test, no release, only xi_U nonzero

    policy1.tauV			= 0;
    policy1.tauS			= 0;
    policy1.xiU				= xiU_c;
    policy1.rU_exp          = rU_exp;
    policy1.d_start_exp 	= Tstar_guess*hrs;       
    policy1.d_start_reopen  = Tstar_guess*hrs; 
    policy1.d_start_testing = Tstar_guess*hrs;     

    experiment              = 'exp1';

    [Reported_D_exp1,Notinfected_D_exp1,Unreported_D_exp1,Infected_D_exp1,Recovered_D_exp1,Dead_D_exp1,Infected_T_exp1,Y_D_exp1,Y_T_exp1,M_T_exp1,A_daily,Pt_exp1,number_tests_D_exp1,Rt_D_exp1]  =corona_func(policy1,model,experiment);
    
    Tstar               = find(Dead_D_exp1>=deaths_March20,1,'first'); 	
    % Be careful, Tstar is in units of "days", so needs to be multiplied by hrs to get to 'model periods' : Tstar*hrs
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %2. Run full simulation, with policy starting on Tstar
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Rerun with correct Tstar 

    d_vaccine               = days_Mar20_to_vaccine*hrs+ Tstar*hrs;    
    % no test, no release, only xi_U nonzero
    policy1.tauV            = 0;
    policy1.tauS            = 0;
    policy1.xiU             = xiU_c;
    policy1.rU              = rU_exp;
    policy1.d_start_exp     =  Tstar*hrs ; 
    policy1.d_start_reopen  =  Tstar*hrs + days_Mar20_to_reopen*hrs;     
    policy1.d_start_testing =  Tstar*hrs + days_Mar20_to_testing*hrs;     

    experiment              = 'exp1';
    clc;    
    [Reported_D_exp1,Notinfected_D_exp1,Unreported_D_exp1,Infected_D_exp1,Recovered_D_exp1,Dead_D_exp1,Infected_T_exp1,Y_D_exp1,Y_T_exp1,M_T_exp1,A_daily,Pt_exp1,number_tests_D_exp1,Rt_D_exp1]  =corona_func(policy1,model,experiment);
    Tstar_check         = find(Dead_D_exp1>=deaths_March20,1,'first'); 			% Be careful, Tstar is in units of "days", so needs to be multiplied by hrs to get to 'model periods' : Tstar*hrs
    assert(Tstar==Tstar_check)


days_model    = [1:1:700]';
days_model    = days_model - days_model(find(Dead_D_exp1>=deaths_March20,1,'first'));

target_deaths = Dead_D_exp1(d_conv);

%% Fig4 - Constant Deaths,  computation part

tau                 = 1/(7*hrs*0.999*weeks);    % constant hourly testing rate

% exp2 only V test  
policy2.tauV                = tau;
policy2.tauS                = 0;
policy2.xiU                 = xiU_c;
policy2.d_start_exp         = policy1.d_start_exp ;
policy2.d_start_reopen      = policy1.d_start_reopen;
policy2.d_start_testing     = policy1.d_start_testing;     

% rU to be specified in function rU_bisection 
experiment             = 'exp2';
constant               = 'Deaths' ;
out2_deaths            = rU_bisection(constant,target_deaths,policy2,model,experiment) ;
assert(out2_deaths.bisection.nonconverge==0)

% exp3 only S test
  
policy3.tauV                = 0;
policy3.tauS                = tau;
policy3.xiU                 = xiU_c;
policy3.d_start_exp         = policy1.d_start_exp ;
policy3.d_start_reopen      = policy1.d_start_reopen;
policy3.d_start_testing     = policy1.d_start_testing;     

experiment       	= 'exp3';
constant            = 'Deaths' ;
out3_deaths         = rU_bisection(constant,target_deaths,policy3,model,experiment) ;
assert(out3_deaths.bisection.nonconverge==0)

eval(['save robustness_' num2str(robust) '_round_' num2str(round)])

out   = [IFR rel_rho psi gamma sigma R0 rU_start];

days_for_output = days(vaccine_date - reopen_date);  % Jun1 to date_end

sum_Y1          = sum(Y_D_exp1((policy1.d_start_reopen/hrs):1:(policy1.d_start_reopen/hrs+days_for_output))-Y_T_exp1(1)) ;  % used Y_T(1) instead of Y_D(1) since Y_T(1) is the *very* initial output
numerator2      = sum(out2_deaths.exp.Y_T(1) - out2_deaths.exp.Y_D(policy2.d_start_reopen/hrs:1:(policy2.d_start_reopen/hrs+days_for_output)) );
outputloss      = numerator2/(-sum_Y1) ;
outputlosssaved = 1-outputloss ;

